﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Goldammer;

namespace Online_PIDController {
    public partial class Form1 : Form {
        public Form1() {
            InitializeComponent();
        }

        MAIDevice m_firstDev = null;
        MAIMeasurementChannelAD m_channel = null;
        bool m_bMeasurement = false;
        double P;
        double I;
        double D;
        double Nom;
        double Gain;
        double Min;
        double Max;

        private void GetPID() {
            P = Convert.ToDouble(textBoxP.Text);
            I = Convert.ToDouble(textBoxI.Text);
            D = Convert.ToDouble(textBoxD.Text);

        }
        private void GetNom() {
            Nom = Convert.ToDouble(textBoxNom.Text);   
        }
        private void GetGainMinMax() {
            Gain = Convert.ToDouble(textBoxGain.Text);
            Min = Convert.ToDouble(textBoxMin.Text);
            Max = Convert.ToDouble(textBoxMax.Text);   

        }

        private void Form1_Load(object sender, EventArgs e) {
            m_firstDev = MAI.Devices[0];
            if (m_firstDev == null) {
                MessageBox.Show("No Card found.");
                Close();
            }
            if (!m_firstDev.HasSoftwareExtension(SWExtension.Online_PID)) {
                MessageBox.Show("This Card does not support PID.");
                Close();
            }
            m_firstDev.ADChannels.ProcessDataBuffers += new MAIMeasurementBufferEvent(ADChannels_ProcessDataBuffers);
            comboBox1.SelectedIndex = 0;

        }

        void ADChannels_ProcessDataBuffers(object obj, EventArgs args) {
            try {
                SetTextboxText("");

                uint NofValues = m_firstDev.ADChannels.GetNumberOfValues();
                double[] Values = null;
                foreach (MAIMeasurementChannelAD c in m_firstDev.ADChannels.GetMeasurementChannels()) {
                    Values = c.ReadData(NofValues);
                }
                if (Values == null)
                    return;
                string textboxtxt = "";
                for (int i = 0; i < 10; i++) {
                    if (i >= Values.Length)
                        return;
                    textboxtxt += Values[i].ToString() + Environment.NewLine;
                }
                SetTextboxText(textboxtxt);

            }
            catch (Exception ex) {
                MessageBox.Show(ex.Message);
            }
        }

        private void SetTextboxText(string AITextboxText) {
            if (InvokeRequired) { 
                MethodInvoker invoker = new MethodInvoker(delegate() {
                    textBox1.Text = AITextboxText;
                });
                Invoke(invoker);
            }
            else {
                textBox1.Text = AITextboxText;
            }
        }


        private void button1_Click(object sender, EventArgs e) {
            if (m_firstDev == null)
                return;
            m_firstDev.ADChannels.StopMeasurement();
            m_firstDev.ADChannels.StopNotificationThread();
            m_firstDev.ClearAllChannelLists();
            buttonChangeNom.Enabled = false;
            buttonChangePIDs.Enabled = false;

            if (!m_bMeasurement) {
                try{
                    GetPID();
                    GetNom();
                    GetGainMinMax();
                    m_channel = m_firstDev.ADChannels[0].CreateMeasurementChannel(OversamplingMode.Disabled, GainFactor.Disabled, true);

                    m_channel.Realtime.AddPIDController(P, I, D, (PIDControllerType)comboBox1.SelectedIndex, Gain, Nom, Min, Max, 0, 1000);

                    m_firstDev.ADChannels.SetSampleRate(1000);
                    m_firstDev.ADChannels.SetMeasurementBufferNotificationLevel(100);
                    m_firstDev.ConfigMeasure();
                    m_firstDev.ADChannels.StartNotificationThread();
                    m_firstDev.ADChannels.StartMeasurement();
                    m_bMeasurement = true;
                    button1.Text = "Stop";
                    buttonChangeNom.Enabled = true;
                    buttonChangePIDs.Enabled = true;
                }
                catch(Exception ex){
                    m_bMeasurement = false;
                    button1.Text = "Start";
                    m_firstDev.ADChannels.StopMeasurement();
                    m_firstDev.ADChannels.StopNotificationThread();
                    m_firstDev.ClearAllChannelLists();
                    MessageBox.Show( ex.Message);
                }
            }
            else {
                m_bMeasurement = false;
                button1.Text = "Start";

            }


        }

        private void buttonChangeNom_Click(object sender, EventArgs e) {
            if (m_channel == null)
                return;
            try {
                GetNom();
                (m_channel.Realtime[0] as MAIRealtimeFunctionPID).ChangeNominalValue(Nom);
            }
            catch (Exception ex) {
                MessageBox.Show(ex.Message);
            }
        }

        private void buttonChangePIDs_Click(object sender, EventArgs e) {
            if (m_channel == null)
                return;
            try {
                GetPID();
                (m_channel.Realtime[0] as MAIRealtimeFunctionPID).ChangePIDParameter(P,I,D);
            }
            catch (Exception ex) {
                MessageBox.Show(ex.Message);
            }
        }

  

 

    }
}
