﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Goldammer;

namespace Online_Trigger {
    public partial class Form1 : Form {
        public Form1() {
            InitializeComponent();
        }

        MAIDevice m_firstDev = null;
        bool m_Measure = false;

        private void Form1_Load(object sender, EventArgs e) {
            m_firstDev = MAI.Devices[0];
            if (m_firstDev == null) {
                MessageBox.Show("No Card found.");
                Close();
            }
            if (!m_firstDev.HasSoftwareExtension(SWExtension.Online_PID)) {
                MessageBox.Show("This Card does not support Trigger Functions.");
                Close();
            }
            m_firstDev.ADChannels.ProcessDataBuffers += new MAIMeasurementBufferEvent(ADChannels_ProcessDataBuffers);

            InitComboboxes();
        }

        RealtimeSetupTrigger GetTrigger1() {
            try {
                RealtimeSetupTrigger t;
                t.Aktiv = checkBoxStartTriggerActive.Checked;
                t.EnabledAtStart = checkBoxStartTriggerEnabled.Checked;
                t.Schwellwert1 = Convert.ToDouble(textBoxstartTrigtresh1.Text);
                t.Schwellwert2 = Convert.ToDouble(textBoxstartTrigtresh2.Text);
                t.TriggerSetztTTL = checkBoxStartSetsTTL.Checked;
                t.TTLKlemme = Convert.ToUInt32(textBoxTTLChannelStart.Text);
                if (m_firstDev != null)
                    m_firstDev.TTLChannels[(int)t.TTLKlemme].SetChannelDirection(IODirection.Output);
                t.TTLStartZustand = (IOBitState)comboBoxStartTTLInit.SelectedIndex;
                t.TTLTriggeredZustand = (IOBitState)comboBoxStartTTLTriggered.SelectedIndex;
                t.Typ = (TriggerType)comboBoxMode1.SelectedIndex;
                t.TriggerStartetMessung = checkBoxStartTriggerMeasure.Checked;
                return t;
            }
            catch(Exception) {
                MessageBox.Show("Invalid number in Trigger 1");
                throw;
            }
        }
        RealtimeSetupTrigger GetTrigger2() {
            try {
                RealtimeSetupTrigger t;
                t.Aktiv = checkBoxStopTriggerActive.Checked;
                t.EnabledAtStart = checkBoxStopTriggerEnable.Checked;
                t.Schwellwert1 = Convert.ToDouble(textBoxStopTrigtresh1.Text);
                t.Schwellwert2 = Convert.ToDouble(textBoxStopTrigtresh2.Text);
                t.TriggerSetztTTL = checkBoxStopSetsTTL.Checked;
                t.TTLKlemme = Convert.ToUInt32(textBoxTTLChannelStop.Text);
                if (m_firstDev != null)
                    m_firstDev.TTLChannels[(int)t.TTLKlemme].SetChannelDirection(IODirection.Output);
                t.TTLStartZustand = (IOBitState)comboBoxStopTTLInit.SelectedIndex;
                t.TTLTriggeredZustand = (IOBitState)comboBoxStopTTLTriggered.SelectedIndex;
                t.Typ = (TriggerType)comboBoxMode2.SelectedIndex;
                t.TriggerStartetMessung = checkBoxStopTriggerMeas.Checked;
                return t;
            }
            catch(Exception) {
                MessageBox.Show("Invalid number in Trigger 2");
                throw;
            }
        }
        private void InitComboboxes() {
            comboBoxMode1.SelectedIndex = 3;
            comboBoxMode2.SelectedIndex = 2;
            comboBoxStartTTLInit.SelectedIndex = 0;
            comboBoxStartTTLTriggered.SelectedIndex = 1;
            comboBoxStopTTLInit.SelectedIndex = 1;
            comboBoxStopTTLTriggered.SelectedIndex = 0;
        }

        void ADChannels_ProcessDataBuffers(object obj, EventArgs args) {
            try {
                SetTextboxText("");

                uint NofValues = m_firstDev.ADChannels.GetNumberOfValues();
                double[] Values = null;
                foreach (MAIMeasurementChannelAD c in m_firstDev.ADChannels.GetMeasurementChannels()) {
                    Values = c.ReadData(NofValues);
                }
                if (Values == null)
                    return;
                string textboxtxt = "";
                for (int i = 0; i < 10; i++) {
                    if (i >= Values.Length)
                        return;
                    textboxtxt += Values[i].ToString() + Environment.NewLine;
                }
                SetTextboxText(textboxtxt);

            }
            catch (Exception ex) {
                MessageBox.Show(ex.Message);
            }
        }

        private void SetTextboxText(string AITextboxText) {
            if (InvokeRequired) {  // muss den Main-Thread "invoken", der Aufruf passiert ja aus dem Notification Thread heraus.
                MethodInvoker invoker = new MethodInvoker(delegate() {
                    textBox1.Text = AITextboxText;
                });
                Invoke(invoker);
            }
            else {
                textBox1.Text = AITextboxText;
            }
        }

        private void trackBar1_Scroll(object sender, EventArgs e) {
            if (m_firstDev == null)
                return;
            if (m_firstDev.DAChannels.HardwareChannelCount < 1)
                return;
            double NuVal = ((double)trackBar1.Value) / 100.0;
            labelDALastValue.Text = NuVal.ToString();
            try {
                m_firstDev.DAChannels[0].WriteChannelVoltage(NuVal);
            }
            catch (Exception ex) {
                MessageBox.Show(ex.Message);
            }
        }

        private void buttonStart_Click(object sender, EventArgs e) {
            if (m_firstDev == null)
                return;
            m_firstDev.StopMeasure();
            m_firstDev.ADChannels.StopNotificationThread();
            m_firstDev.ClearAllChannelLists();
            try {
                if (!m_Measure) {

                    MAIMeasurementChannelAD c =  m_firstDev.ADChannels[0].CreateMeasurementChannel(OversamplingMode.Disabled, GainFactor.Disabled, true);
                    m_firstDev.ADChannels.SetSampleRate(250);
                    m_firstDev.ADChannels.SetMeasurementBufferNotificationLevel(10);
                    c.Realtime.AddTrigger(GetTrigger1(), GetTrigger2(), checkBoxReTrigger.Checked);
                    m_firstDev.ADChannels.SetPrePostTriggerValues(0, 0, true);
                    m_firstDev.ConfigMeasure();
                    m_firstDev.ADChannels.StartNotificationThread();
                    m_firstDev.ADChannels.StartMeasurement();

                    groupBox1.Enabled = false;
                    groupBox2.Enabled = false;
                    buttonStart.Text = "Stop";
                    m_Measure = true;

                }
                else {
                    groupBox1.Enabled = true;
                    groupBox2.Enabled = true;
                    buttonStart.Text = "Start";
                    m_Measure = false;
                }
            }
            catch (Exception ex) {
                MessageBox.Show(ex.Message);
                m_firstDev.StopMeasure();
                m_firstDev.ADChannels.StopNotificationThread();
                m_firstDev.ClearAllChannelLists();
                groupBox1.Enabled = true;
                groupBox2.Enabled = true;
                buttonStart.Text = "Start";
                m_Measure = false;

            }


        }
 

    }
}
